import os
import sys

try:
    from unittest import mock
except ImportError:
    import mock

TESTDIR = os.path.dirname(os.path.abspath(__file__))
MAINDIR = os.path.dirname(TESTDIR)
DOCSDIR = os.path.join(MAINDIR, "docs")
DATADIR = os.path.join(TESTDIR, "resources")
RUNDIR = os.path.join(TESTDIR, "run")

# Force importing the local version of the module
sys.path.insert(0, MAINDIR)

# Launch a stub HTTP server to server local files
from .stubs import StubHTTPServer
StubHTTPServer(DATADIR).start()



# coding: utf-8
import threading
import os

try:
    from http.server import HTTPServer
    from http.server import SimpleHTTPRequestHandler
except ImportError:
    from BaseHTTPServer import HTTPServer
    from SimpleHTTPServer import SimpleHTTPRequestHandler

class StubHTTPServer(threading.Thread):
    daemon = True

    class _RequestHandler(SimpleHTTPRequestHandler):
        def translate_path(self, path):
            return self._datadir + path
        def log_message(self, format, *args):
            pass

    def __init__(self, datadir):
        super(StubHTTPServer, self).__init__()
        self._RequestHandler._datadir = datadir
        self.server = HTTPServer(("localhost", 8080), self._RequestHandler)

    def run(self):
        self.server.serve_forever()



# coding: utf-8

import sys
import six
import os
import unittest
import warnings

from . import utils
import pronto


class TestProntoUtils(unittest.TestCase):

    def test_output_str(self):
        """Test function output_str provided by pronto.utils
        """
        @pronto.utils.output_str
        def unicode_returning_function():
            """This function returns str in Py3 and unicode in Py2"""
            return six.u("abc")

        self.assertIsInstance(unicode_returning_function(), str)

    def test_unique_everseen(self):
        """Test function unique_everseen provided by pronto.utils
        """
        self.assertEqual(['a', 'b', 'c', 'd'], list(pronto.utils.unique_everseen("aaaabbbcccaaadd")))

    def test_owlparser_get_id_from_url(self):
        """Test method _get_id_from_url of pronto.parser.owl.OwlXMLParser
        """
        test_values = {
            'http://purl.obolibrary.org/obo/IAO_0000115': 'IAO:0000115',
            'http://purl.obolibrary.org/obo/CL_0002420': 'CL:0002420',
            'http://purl.obolibrary.org/obo/UBERON_0001278': 'UBERON:0001278',
            'http://nmrML.org/nmrCV#NMR:1000001': 'NMR:1000001',
        }
        for k,v in six.iteritems(test_values):
            self.assertEqual(pronto.parser.owl.OwlXMLParser._get_id_from_url(k), v)


def setUpModule():
    warnings.simplefilter('ignore')

def tearDownModule():
    warnings.simplefilter(warnings.defaultaction)



# coding: utf-8
"""
pronto.parser.obo
=================

This module defines the Obo parsing method.
"""
from __future__ import absolute_import
from __future__ import unicode_literals

import collections
import six
import string

from .              import Parser
from .utils         import OboSection
from ..synonym      import SynonymType, Synonym
from ..relationship import Relationship
from ..term         import Term

_obo_synonyms_map = {'exact_synonym': 'EXACT', 'broad_synonym': 'BROAD',
                    'narrow_synonym': 'NARROW', 'synonym': 'RELATED'}

class OboParser(Parser):

    extensions = (".obo", ".obo.gz")

    def hook(self, **kwargs):
        """Returns True if this parser should be used.

        The current behaviour relies on filenames and file extension
        (.obo), but this is subject to change.
        """
        return kwargs.get('force', False) \
            or kwargs.get('path', '').endswith(self.extensions)

    @classmethod
    def parse(cls, stream):
        """Parse the stream.

        Parameters:
            stream (file handle): a binary stream of the file to parse

        Returns:
            dict: a dictionary containing the metadata headers
            dict: a dictionnary containing the terms
            set:  a set containing the imports

        """
        _section    = OboSection.meta
        meta        = collections.defaultdict(list)
        _rawterms   = []
        _rawtypedef = []

        _term_parser = cls._parse_term(_rawterms)
        next(_term_parser)

        for streamline in stream:

            # manage encoding && cleaning of line
            streamline = streamline.decode('utf-8')
            if streamline[0] in string.whitespace:
                continue
            elif streamline[0] == "[":
                _section = cls._check_section(streamline, _section)

            if _section is OboSection.meta:
                cls._parse_metadata(streamline, meta)
            elif _section is OboSection.typedef:
                cls._parse_typedef(streamline, _rawtypedef)
            elif _section is OboSection.term:
                _term_parser.send(streamline)
                #_rawterms = cls._parse_term(streamline, _rawterms)

        terms = cls._classify(_rawtypedef, _rawterms)
        imports = set(meta['import']) if 'import' in meta else set()

        return dict(meta), terms, imports

    @staticmethod
    def _check_section(line, section):
        """Updates the section currently parsed

        The parser starts in the OboSection.meta section but once
        it reaches the first [Typedef], it will enter the OboSection.typedef
        section, and/or when it reaches the first [Term], it will enter
        the OboSection.term section.
        """
        if "[Term]" in line:
            section = OboSection.term
        elif "[Typedef]" in line:
            section = OboSection.typedef
        return section

    @classmethod
    def _parse_metadata(cls, line, meta, parse_remarks=True):
        """Parse a metadata line

        The metadata is organized as a "key: value" statement which
        is split into the proper key and the proper value.

        Parameters:
            line (str): the line containing the metadata
            parse_remarks(bool): if the remarks should be parsed as well
                (read the note) [default: True]

        Note:
            If the line follows the following schema: "remark: key: value",
            the function will attempt to extract the proper key/value
            instead of leaving everything inside the remark key.

            This may cause issues when the line is identified as such even
            though the remark is simply a sentence containing a colon, such as
            "remark: 090506 "Attribute" in Term deleted and new entries: Scan Type [...]"
            (found in imagingMS.obo). To prevent the splitting from happening,
            the text on the left of the colon must be less that *20 chars long*.
        """
        key, value = line.split(':', 1)
        key, value = key.strip(), value.strip()
        if parse_remarks and "remark" in key:                        # Checking that the ':' is not
            if 0<value.find(': ')<20:                                # not too far avoid parsing a sentence
                try:                                                 # containing a ':' as a key: value
                    cls._parse_metadata(value, meta, parse_remarks)  # obo statement nested in a remark
                except ValueError:                                   # (20 is arbitrary, it may require
                    pass                                             # tweaking)
        else:
            meta[key].append(value)
            try:
                syn_type_def = []
                for m in meta['synonymtypedef']:
                    if not isinstance(m, SynonymType):
                        x = SynonymType.from_obo_header(m)
                        syn_type_def.append(x)
                    else:
                        syn_type_def.append(m)
            except KeyError:
                pass
            else:
                meta['synonymtypedef'] = syn_type_def


    @staticmethod
    def _parse_typedef(line, _rawtypedef):
        """Parse a typedef line

        The typedef is organized as a succesion of key:value pairs
        that are extracted into the same dictionnary until a new
        "[Typedef]" header is encountered

        Parameters:
            line (str): the line containing a typedef statement
        """
        if "[Typedef]" in line:
            _rawtypedef.append(collections.defaultdict(list))
        else:
            key, value = line.split(':', 1)
            _rawtypedef[-1][key.strip()].append(value.strip())
        #return _rawtypedef

    @staticmethod
    def _parse_term(_rawterms):
        """Parse a term line

        The term is organized as a succesion of key:value pairs
        that are extracted into the same dictionnary until a new
        "[Term]" header is encountered

        Parameters:
            line (str): the line containing a term statement
        """
        line = yield
        _rawterms.append(collections.defaultdict(list))
        while True:
            line = yield
            if "[Term]" in line:
                _rawterms.append(collections.defaultdict(list))
            else:
                key, value = line.split(':', 1)
                _rawterms[-1][key.strip()].append(value.strip())
            #_rawterms

    @staticmethod
    def _classify(_rawtypedef, _rawterms):
        """Create proper objects out of the extracted dictionnaries

        New Relationship objects are instantiated with the help of
        the :obj:`Relationship._from_obo_dict` alternate constructor.

        New Term objects are instantiated by manually extracting id,
        name, desc and relationships out of the raw _term dictionnary,
        and then calling the default constructor.
        """
        terms = collections.OrderedDict()
        _cached_synonyms = {}

        for _typedef in _rawtypedef:
            Relationship._from_obo_dict( # instantiate a new Relationship
                {k:v for k,lv in six.iteritems(_typedef) for v in lv}
            )


        for _term in _rawterms:
            synonyms = set()

            _id   = _term['id'][0]
            _name = _term.pop('name', ('',))[0]
            _desc = _term.pop('def', ('',))[0]

            _relations = collections.defaultdict(list)
            try:
                for other in _term.get('is_a', ()):
                    _relations[Relationship('is_a')].append(other.split('!')[0].strip())
            except IndexError:
                pass
            try:
                for relname, other in ( x.split(' ', 1) for x in _term.pop('relationship', ())):
                    _relations[Relationship(relname)].append(other.split('!')[0].strip())
            except IndexError:
                pass

            for key, scope in six.iteritems(_obo_synonyms_map):
                for obo_header in _term.pop(key, ()):
                    try:
                        s = _cached_synonyms.get(obo_header)
                    except KeyError:
                         s = Synonym.from_obo_header(obo_header, scope)
                         _cached_synonyms[obo_header] = s
                    finally:
                        synonyms.add(s)

            terms[_id] = Term(_id, _name, _desc, dict(_relations), synonyms, dict(_term))
        return terms


OboParser()



# coding: utf-8
"""
pronto.relationship
===================

This submodule contains the definition of the :ref:`Synonym`
and :ref:`SynonymType` classes.
"""
from __future__ import unicode_literals
from __future__ import absolute_import

import re
import six
import collections

from .utils import output_str


class SynonymType(object):
    """A custom synonym type (obo-style).

    Attributes:
        name(str): the name of the synonym type
        scope(str, optional): the scope all synonyms of
            that type will always have(either 'EXACT',
            'BROAD', 'NARROW', 'RELATED', or None).
        desc(str): the description of the synonym type
    """
    __slots__ = ['name', 'desc', 'scope']
    _instances = collections.OrderedDict()
    _RX_OBO_EXTRACTER = re.compile(r'(?P<name>[^ ]*)[ ]*\"(?P<desc>.*)\"[ ]*(?P<scope>BROAD|NARROW|EXACT|RELATED)?')

    def __init__(self, name, desc, scope=None):
        self.name = name
        self.desc = desc
        if scope in {'BROAD', 'NARROW', 'EXACT', 'RELATED', None}:
            self.scope = scope
        elif scope in {six.b('BROAD'), six.b('NARROW'), six.b('EXACT'), six.b('RELATED')}:
            self.scope = scope.decode('utf-8')
        else:
            raise ValueError("scope must be 'NARROW', 'BROAD', 'EXACT', 'RELATED' or None")
        self._register()

    def _register(self):
        self._instances[self.name] = self

    @classmethod
    def from_obo_header(cls, obo_header):
        if isinstance(obo_header, six.binary_type):
            obo_header = obo_header.decode('utf-8')

        groupdict = cls._RX_OBO_EXTRACTER.search(obo_header).groupdict()
        result = {k:v.strip() if v else None for k,v in six.iteritems(groupdict)}
        return cls(**result)

    @property
    def obo(self):
        return ' '.join(['synonymtypedef:', self.name,
                         '"{}"'.format(self.desc),
                         self.scope or '']).strip()

    @output_str
    def __repr__(self):
        return ''.join(['<SynonymType: ', self.name, ' ',
                        '"{}"'.format(self.desc),
                        ' {}>'.format(self.scope) \
                        if self.scope else '>']).strip()

    def __hash__(self):
        return hash((self.name, self.desc, self.scope))


class Synonym(object):
    """A synonym representation (obo-like).

    Attributes:
        desc (str): a description of the synonym
        syn_type (SynonymType, optional): the type of synonym if relying
            on a custom type defined in the ontology metadata
        scope (str, optional): the scope of the synonym (either EXACT,
            BROAD, NARROW or RELATED).
        xref (list, optional): the list of the cross-references of
            the synonym
    """
    _RX_OBO_EXTRACTER = re.compile(r'\"(?P<desc>.*)\" *(?P<scope>EXACT|BROAD|NARROW|RELATED)? *(?P<syn_type>[^ ]+)? \[(?P<xref>.*)\]')

    def __init__(self, desc, scope=None, syn_type=None, xref=None):

        if isinstance(desc, six.binary_type):
            self.desc = desc.decode('utf-8')
        elif isinstance(desc, six.text_type):
            self.desc = desc
        else:
            raise ValueError("desc must be bytes or str, not {}".format(type(desc).__name__))

        if isinstance(scope, six.binary_type):
            self.scope = scope.decode('utf-8')
        elif isinstance(scope, six.text_type):
            self.scope = scope
        elif scope is None:
            self.scope = "RELATED"

        if syn_type is not None:
            try:
                self.syn_type = SynonymType._instances[syn_type]
                self.scope = self.syn_type.scope or self.scope or 'RELATED'
            except KeyError as e:
                raise ValueError("Undefined synonym type: {}".format(syn_type))
        else:
            self.syn_type = None

        if self.scope not in {'EXACT', 'BROAD', 'NARROW', 'RELATED', None}:
            raise ValueError("scope must be 'NARROW', 'BROAD', 'EXACT', 'RELATED' or None")

        self.xref = xref or []

    @classmethod
    def from_obo_header(cls, obo_header, scope='RELATED'):

        if isinstance(obo_header, six.binary_type):
            obo_header = obo_header.decode('utf-8')

        groupdict = cls._RX_OBO_EXTRACTER.search(obo_header).groupdict()
        if groupdict.get('xref', ''):
            groupdict['xref'] = [x.strip() for x in groupdict['xref'].split(',')]
        groupdict['syn_type'] = groupdict['syn_type'] or None

        # result = {k:v.strip() if v else None for k,v in six.iteritems(groupdict)}
        # if result['xref'] is not None:
        #     result['xref'] = [x.strip() for x in result['xref'].split(',')]
        return cls(**groupdict)#cls(**result)

    @property
    def obo(self):
        return 'synonym: "{}" {} [{}]'.format(
            self.desc,
            ' '.join([self.scope, self.syn_type])\
                if self.syn_type else self.scope,
            ', '.join(self.xref)
        )

    @output_str
    def __repr__(self):
        return '<Synonym: "{}" {} [{}]>'.format(
            self.desc,
            ' '.join([self.scope, self.syn_type.name])\
                if self.syn_type else self.scope,
            ', '.join(self.xref)
        )

    def __eq__(self, other):
        return self.desc==other.desc and self.scope==other.scope and self.syn_type==other.syn_type and self.xref==other.xref

    def __hash__(self):
        return hash((self.desc, self.scope, self.syn_type, tuple(self.xref)))



# coding: utf-8
from __future__ import absolute_import

import unittest
import os
import six
import sys
import warnings
import shutil

try:
    import sphinx
except ImportError:
    sphinx = None

from . import utils
import pronto


### TESTS ###
class TestProntoDocumentation(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.build_dir = os.path.join(utils.TESTDIR, "run", "build")
        cls.source_dir = os.path.join(utils.DOCSDIR, "source")
        os.mkdir(os.path.join(utils.TESTDIR, "run"))

    @classmethod
    def tearDownClass(cls):
        shutil.rmtree(os.path.join(utils.TESTDIR, "run"))

    @classmethod
    @unittest.skipUnless(sys.version[:3] in {'3.6', '2.7'},
        "Python {} cannot run Sphinx".format(sys.version[:3]))
    def register_tests(cls):
        """Register tests for most sphinx builders"""
        for builder in ('html', 'latexpdf', 'json', 'man', 'xml'):
            cls.add_test(builder)

    @classmethod
    def add_test(cls, builder):
        def _test(self):
            with utils.mock.patch('sys.stderr', six.moves.StringIO()) as stderr:
                with utils.mock.patch('sys.stdout', six.moves.StringIO()) as stdout:
                    self.assertEquals(0, sphinx.build_main([
                        "-b{}".format(builder),
                        "-d{}".format(os.path.join(self.build_dir, 'doctrees')),
                        self.source_dir,
                        os.path.join(self.build_dir, builder)])
                    )
        setattr(cls, "test_{}".format(builder), _test)


def setUpModule():
    warnings.simplefilter('ignore')

def tearDownModule():
    warnings.simplefilter(warnings.defaultaction)

def load_tests(loader, tests, pattern):
    suite = unittest.TestSuite()
    #TestProntoDocumentation.register_tests()
    suite.addTests(loader.loadTestsFromTestCase(TestProntoDocumentation))
    return suite



# coding: utf-8
from __future__ import absolute_import

### DEPS
import six
import unittest
import io
import sys
import contextlib
import os
import shutil
import gzip
import os.path as op
import warnings
import textwrap

from . import utils
import pronto.synonym


class TestProntoSynonymType(unittest.TestCase):

    def assertOk(self, synonymtype, scope):
        self.assertEqual(synonymtype.desc, 'British spelling')
        self.assertEqual(synonymtype.name, 'UK_SPELLING')
        self.assertEqual(synonymtype.scope, scope)
        self.assertIn(synonymtype.name, pronto.synonym.SynonymType._instances)

    def test_new_synonymtype_from_obo_header_with_scope(self):
        synonymtype = pronto.synonym.SynonymType.from_obo_header(
            'UK_SPELLING "British spelling" EXACT'
        )
        self.assertOk(synonymtype, "EXACT")

    def test_new_synonymtype_from_obo_header_without_scope(self):
        synonymtype = pronto.synonym.SynonymType.from_obo_header(
            'UK_SPELLING "British spelling"'
        )
        self.assertOk(synonymtype, None)

    def test_new_synonymtype_with_scope(self):
        synonymtype = pronto.synonym.SynonymType(
            'UK_SPELLING', 'British spelling', 'EXACT',
        )
        self.assertOk(synonymtype, "EXACT")

    def test_new_synonymtype_without_scope(self):
        synonymtype = pronto.synonym.SynonymType(
            'UK_SPELLING', 'British spelling',
        )
        self.assertOk(synonymtype, None)

    def test_fail_wrong_scope(self):
        with self.assertRaises(ValueError) as ctx:
            pronto.synonym.SynonymType(
            'UK_SPELLING', 'British spelling', 'UNEXISTING_SCOPE',
        )
        self.assertEqual(str(ctx.exception), "scope must be 'NARROW'"
                         ", 'BROAD', 'EXACT', 'RELATED' or None")

    def test_obo_with_scope(self):
        obo_header = 'UK_SPELLING "British spelling" EXACT'
        synonymtype = pronto.synonym.SynonymType.from_obo_header(obo_header)
        self.assertEqual(synonymtype.obo, "synonymtypedef: {}".format(obo_header))

    def test_obo_without_scope(self):
        obo_header = 'UK_SPELLING "British spelling"'
        synonymtype = pronto.synonym.SynonymType.from_obo_header(obo_header)
        self.assertEqual(synonymtype.obo, "synonymtypedef: {}".format(obo_header))


class TestProntoSynonym(unittest.TestCase):

    def tearDown(self):
        pronto.synonym.SynonymType._instances.clear()

    def assertOk(self, synonym, scope, synonymtype=None):
        self.assertEqual(synonym.desc, "The other white meat")
        self.assertEqual(synonym.scope, scope)
        self.assertEqual(synonym.xref, ['MEAT:00324', 'BACONBASE:03021'])
        if synonymtype is not None:
            self.assertEqual(synonym.syn_type, synonymtype)

    def test_new_synonym_from_obo_header_with_scope_with_syntype(self):
        synonymtype = pronto.synonym.SynonymType("MARKETING_SLOGAN", 'marketing slogan')
        synonym = pronto.synonym.Synonym.from_obo_header(
            '"The other white meat" EXACT MARKETING_SLOGAN [MEAT:00324, BACONBASE:03021]'
        )
        self.assertOk(synonym, "EXACT", synonymtype)

    def test_new_synonym_from_obo_header_without_scope_with_syntype(self):
        synonymtype = pronto.synonym.SynonymType("MARKETING_SLOGAN", 'marketing slogan')
        synonym = pronto.synonym.Synonym.from_obo_header(
            '"The other white meat" MARKETING_SLOGAN [MEAT:00324, BACONBASE:03021]'
        )
        self.assertOk(synonym, "RELATED", synonymtype)

    def test_new_synonym_with_scope_with_syntype(self):
        synonymtype = pronto.synonym.SynonymType("MARKETING_SLOGAN", 'marketing slogan')
        synonym = pronto.synonym.Synonym(
            "The other white meat", "BROAD", "MARKETING_SLOGAN", ["MEAT:00324", "BACONBASE:03021"],
        )
        self.assertOk(synonym, "BROAD", synonymtype)

    def test_new_synonym_without_scope_with_syntype(self):
        synonymtype = pronto.synonym.SynonymType("MARKETING_SLOGAN", 'marketing slogan')
        synonym = pronto.synonym.Synonym(
            "The other white meat", None, "MARKETING_SLOGAN", ["MEAT:00324", "BACONBASE:03021"],
        )
        self.assertOk(synonym, "RELATED", synonymtype)

    def test_new_synonym_from_obo_header_with_scope_without_syntype(self):
        synonym = pronto.synonym.Synonym.from_obo_header(
            '"The other white meat" EXACT [MEAT:00324, BACONBASE:03021]'
        )
        self.assertOk(synonym, "EXACT")

    def test_new_synonym_from_obo_header_without_scope_without_syntype(self):
        synonym = pronto.synonym.Synonym.from_obo_header(
            '"The other white meat" [MEAT:00324, BACONBASE:03021]'
        )
        self.assertOk(synonym, "RELATED")

    def test_new_synonym_with_scope_without_syntype(self):
        synonym = pronto.synonym.Synonym(
            "The other white meat", "BROAD", None, ["MEAT:00324", "BACONBASE:03021"],
        )
        self.assertOk(synonym, "BROAD")

    def test_new_synonym_without_scope_without_syntype(self):
        synonym = pronto.synonym.Synonym(
            "The other white meat", None, None, ["MEAT:00324", "BACONBASE:03021"],
        )
        self.assertOk(synonym, "RELATED")

    def test_new_synonym_inherit_scope_from_syn_type(self):
        synonymtype = pronto.synonym.SynonymType("MARKETING_SLOGAN", 'marketing slogan', "EXACT")
        synonym = pronto.synonym.Synonym(
            "The other white meat", None, "MARKETING_SLOGAN", ["MEAT:00324", "BACONBASE:03021"],
        )
        self.assertOk(synonym, "EXACT", synonymtype)

    def test_new_synonym_force_scope_from_syn_type(self):
        synonymtype = pronto.synonym.SynonymType("MARKETING_SLOGAN", 'marketing slogan', "EXACT")
        synonym = pronto.synonym.Synonym(
            "The other white meat", "BROAD", "MARKETING_SLOGAN", ["MEAT:00324", "BACONBASE:03021"],
        )
        self.assertOk(synonym, "EXACT", synonymtype)

    def test_fail_wrong_scope(self):
        synonymtype = pronto.synonym.SynonymType("MARKETING_SLOGAN", 'marketing slogan')
        with self.assertRaises(ValueError) as ctx:
            synonym = pronto.synonym.Synonym(
                "The other white meat", "WRONG", "MARKETING_SLOGAN", ["MEAT:00324", "BACONBASE:03021"],
        )

    def test_fail_undefined_syn_type(self):
        with self.assertRaises(ValueError) as ctx:
            synonym = pronto.synonym.Synonym(
                "The other white meat", "BROAD", "WRONG_TYPE", ["MEAT:00324", "BACONBASE:03021"],
        )



# coding: utf-8
import sys
import six
import abc
import os
import time
import functools
import unittest
import importlib
import warnings
import platform
import gzip

from . import utils
import pronto




class TestProntoParser(unittest.TestCase):

    def setUp(self):
        self.resources_dir = os.path.join(utils.TESTDIR, "resources")

    def _check(self, m,t,i, *args, **kwargs):
        #FEAT# Store internals as utf-8
        #self._check_utf8(m,t,i)

        if 'exp_len' in kwargs:
            self._check_len(t, kwargs['exp_len'])

        for term in six.itervalues(t):
            self._check_synonyms(term)

    def _check_synonyms(self, term):
        """Check that each Term synonym is stored as Synonym object
        """
        for s in term.synonyms:
            self.assertIsInstance(s, pronto.synonym.Synonym)

    def _check_utf8(self, m, t, i):
        """Check that inner properties are stored as utf-8
        """
        for term in six.itervalues(t):
            # six.text_type is str in Py3, unicode in Py2
            self.assertIsInstance(term.id, six.text_type)
            self.assertIsInstance(term.name, six.text_type)
            self.assertIsInstance(term.desc, six.text_type)

        for importpath in i:
            self.assertIsInstance(importpath, six.text_type)

        for k,v in six.iteritems(m):
            self.assertIsInstance(k, six.text_type)
            for x in v:
                self.assertIsInstance(x, six.text_type)

    def _check_len(self, t, exp_len):
        """Check the terms dict length matches the expected length
        """
        self.assertEqual(len(t), exp_len)

    @staticmethod
    def _parse(parser, path):
        if path.startswith(('http', 'ftp')):
            handle = six.moves.urllib.request.urlopen(path)
        elif path.endswith('.gz'):
            handle = gzip.GzipFile(path)
        else:
            handle = open(path, 'rb')
        try:
            m,t,i = parser.parse(handle)
        finally:
            handle.close()
        return m,t,i



@six.add_metaclass(abc.ABCMeta)
class TestProntoOwlParser(object):

    parser = None
    imp = platform.python_implementation()

    def setUp(self):
        super(TestProntoOwlParser, self).setUp()
        self.lxml_etree = importlib.import_module("lxml.etree")
        self.xml_etree = importlib.import_module("xml.etree.ElementTree")
        self.cxml_etree = importlib.import_module("xml.etree.cElementTree")

    # ----------------------------------------
    # Test importing file with inline comments
    # ----------------------------------------

    @unittest.skipIf(imp != "CPython", 'cannot run lxml on {}'.format(imp))
    def test_inline_comment_lxml(self):
        try:
            with utils.mock.patch("pronto.parser.owl.etree", self.lxml_etree):
                tree_m, tree_t, tree_i = self._parse(
                    pronto.parser.owl.OwlXMLTreeParser(),
                    os.path.join(self.resources_dir, 'obi-small.owl'),
                )
        except Exception as e:
            self.fail(e)

    @unittest.skipIf(imp != "CPython", 'cannot run cElementTree on {}'.format(imp))
    def test_inline_comment_cElementTree(self):
        try:
            with utils.mock.patch("pronto.parser.owl.etree", self.cxml_etree):
                tree_m, tree_t, tree_i = self._parse(
                    pronto.parser.owl.OwlXMLTreeParser(),
                    os.path.join(self.resources_dir, 'obi-small.owl'),
                )
        except Exception as e:
            self.fail(e)

    def test_inline_comment_ElementTree(self):
        try:
            with utils.mock.patch("pronto.parser.owl.etree", self.xml_etree):
                tree_m, tree_t, tree_i = self._parse(
                    pronto.parser.owl.OwlXMLTreeParser(),
                    os.path.join(self.resources_dir, 'obi-small.owl'),
                )
        except Exception as e:
            self.fail(e)

    # -----------------------------------
    # Test importing from "local" source
    # -----------------------------------

    @unittest.skipIf(imp != "CPython", 'cannot run lxml on {}'.format(imp))
    def test_with_lxml_etree(self):
        with utils.mock.patch("pronto.parser.owl.etree", self.lxml_etree):
            m,t,i = self._parse(
                self.parser(),
                os.path.join(self.resources_dir, 'cl.ont.gz'),
            )
        self._check(m,t,i, exp_len=2348)

    @unittest.skipIf(imp != "CPython", 'cannot run cElementTree on {}'.format(imp))
    def test_with_xml_cElementTree(self):
        with utils.mock.patch("pronto.parser.owl.etree", self.cxml_etree):
            m,t,i = self._parse(
                self.parser(),
                os.path.join(self.resources_dir, 'cl.ont.gz'),
            )
        self._check(m,t,i, exp_len=2348)

    def test_with_xml_elementTree(self):
        with utils.mock.patch("pronto.parser.owl.etree", self.xml_etree):
            m,t,i = self._parse(
                self.parser(),
                os.path.join(self.resources_dir, 'cl.ont.gz'),
            )
        self._check(m,t,i, exp_len=2348)

    # -----------------------------------
    # Test importing from "remote" source
    # -----------------------------------

    @unittest.skipIf(imp != "CPython", 'cannot run lxml on {}'.format(imp))
    def test_with_lxml_etree_remote(self):
        with utils.mock.patch("pronto.parser.owl.etree", self.lxml_etree):
            m,t,i = self._parse(
                self.parser(),
                "http://localhost:8080/nmrCV.owl",
            )
        self._check(m,t,i, exp_len=685)

    @unittest.skipIf(imp != "CPython", 'cannot run cElementTree on {}'.format(imp))
    def test_with_xml_cElementTree_remote(self):
        with utils.mock.patch("pronto.parser.owl.etree", self.cxml_etree):
            m,t,i =  self._parse(
                self.parser(),
                "http://localhost:8080/nmrCV.owl",

            )
        self._check(m,t,i, exp_len=685)

    def test_with_xml_elementTree_remote(self):
        with utils.mock.patch("pronto.parser.owl.etree", self.xml_etree):
            m,t,i = self._parse(
                self.parser(),
                "http://localhost:8080/nmrCV.owl",
            )
        self._check(m,t,i, exp_len=685)


class TestProntoOwlTargetParser(TestProntoOwlParser, TestProntoParser):
    parser = pronto.parser.owl.OwlXMLTargetParser


class TestProntoOwlTreeParser(TestProntoOwlParser, TestProntoParser):
    parser = pronto.parser.owl.OwlXMLTreeParser


class TestProntoOwlUnicity(TestProntoParser):

    def test_parser_unicity(self):
        tree_m, tree_t, tree_i  = self._parse(
                pronto.parser.owl.OwlXMLTreeParser(),
                os.path.join(self.resources_dir, 'cl.ont.gz'),
            )
        target_m, target_t, target_i = self._parse(
                pronto.parser.owl.OwlXMLTargetParser(),
                os.path.join(self.resources_dir, 'cl.ont.gz'),
            )

        #print(target_i)
        #print(target_m)
        #tree_cl = pronto.Ontology(os.path.join(self.resources_dir, 'cl.ont.gz'), 'OwlXMLTreeParser')
        #target_cl = pronto.Ontology(os.path.join(self.resources_dir, 'cl.ont.gz'), 'OwlXMLTargetParser')

        self.assertEqual(set(tree_t.keys()), set(target_t.keys()))
        self.assertEqual(set(tree_m.keys()), set(target_m.keys()))

        #FEAT# Acceptance test for althonos/pronto#7
        #self.assertEqual(tree_i, target_i)

        # for tid in tree_t:
        #     tree_term = tree_t[tid]
        #     target_term = target_t[tid]
        #     self.assertEqual(tree_term.id, target_term.id)
        #     self.assertEqual(tree_term.name, target_term.name)
        #     self.assertEqual(tree_term.desc, target_term.desc)
        #     self.assertEqual(tree_term.relations.keys(), target_term.relations.keys())
        #     self.assertEqual(tree_term.synonyms, target_term.synonyms)




def setUpModule():
    warnings.simplefilter('ignore')

def tearDownModule():
    warnings.simplefilter(warnings.defaultaction)



# coding: utf-8 from __future__ import absolute_import

### DEPS
import six
import unittest
import io
import sys
import contextlib
import os
import shutil
import gzip
import os.path as op
import warnings
import textwrap

from . import utils
import pronto


### TESTS
class TestProntoOntology(unittest.TestCase):

    def assert_loaded(self, ontology):

        # check if not empty
        self.assertNotEqual(len(ontology), 0)

        # check if succesfully crosslinked
        for term in ontology:
            for l in term.relations.values():
                for other in l:
                    self.assertIsInstance(other, pronto.Term)

    def assert_exportable(self, ontology):
        try:
            file = six.moves.StringIO()
            file.write(ontology.obo)
        except BaseException as e:
            self.fail("export failed: {}".format(e))

    def assert_mergeable(self, ontology):
        try:
            other = pronto.Ontology()
            other.merge(ontology)
        except BaseException as e:
            self.fail("merge failed: {}".format(e))
        self.assertEqual(len(other), len(ontology))

    def check_ontology(self, ontology):
        self.assert_loaded(ontology)
        self.assert_exportable(ontology)
        self.assert_mergeable(ontology)





class TestProntoFeatures(TestProntoOntology):

    @classmethod
    def setUpClass(cls):
        cls.consistency_span = 10

    def test_ims_consistency(self):
        """Assert several runs on the same file give the same output.

        See [https://github.com/althonos/pronto/issues/4]
        Thanks to @winni-genp for issue reporting.
        """
        obo = pronto.Ontology(os.path.join(utils.DATADIR, "imagingMS.obo.gz"), False)
        expected_keys = set(obo.terms.keys())
        for _ in six.moves.range(self.consistency_span):
            tmp_obo = pronto.Ontology(os.path.join(utils.DATADIR, "imagingMS.obo.gz"), False)
            self.assertEqual(set(tmp_obo.terms.keys()), expected_keys)
        self.check_ontology(obo)

    def test_remote_obogz_parsing(self):
        """Test the behaviour of pronto when parsing remote gzipped files.

        On Python2, this should raise a NotImplementedError (as
        urllib2.urlopen does not implement :obj:`io.StringIO` methods,
        gzip.GzipFile cannot handle on-the-fly decoding of data).

        On Python3, this should actually produce a proper ontology.
        """
        hpo_url = "http://localhost:8080/hpo.obo.gz"
        if six.PY3:
            gzipped = pronto.Ontology(hpo_url)
            expected_keys = {
                x.decode('utf-8').strip().split(': ')[-1]
                    for x in gzip.GzipFile(
                        fileobj=six.moves.urllib.request.urlopen(hpo_url)
                    )
                        if x.startswith(b'id: ')
            }
            self.assertEqual(expected_keys, set(gzipped.terms.keys()))
            self.check_ontology(gzipped)
        else:
            with self.assertRaises(NotImplementedError):
                gzipped = pronto.Ontology(hpo_url, parser='OwlXMLTargetParser')

    def test_unicode_in_term_names(self):
        """Test if unicode characters in term names work.

        See [https://github.com/althonos/pronto/issues/6]
        Thanks to @owen-jones-gen for issue reporting.
        """
        ontology = pronto.Ontology("tests/resources/owen-jones-gen.obo")
        with utils.mock.patch('sys.stdout', new=six.moves.StringIO()) as output:
            for term in ontology:
                print(term)
                print(term.obo)
            print(ontology.obo)

            output.seek(0)
            self.assertEqual(output.read().strip(),
                             textwrap.dedent("""
                                             <ONT0:ROOT: °>
                                             [Term]
                                             id: ONT0:ROOT
                                             name: °
                                             [Term]
                                             id: ONT0:ROOT
                                             name: °
                                             """).strip())

        self.check_ontology(ontology)

    def test_exportable_headerless_ontology(self):
        """Test if a headerless ontology can be exported

        See [https://github.com/althonos/pronto/issues/5]
        Thanks to @winni-genp for issue reporting.
        """
        obo = pronto.Ontology("tests/resources/winni-genp.obo")
        self.check_ontology(obo)

    def test_with_custom_typedef(self):
        """Try to import an obo ontology containing custom typedefs
        """
        obo = pronto.Ontology("tests/resources/elo.obo")
        self.check_ontology(obo)

        self.assertIn("has_written", pronto.Relationship._instances)
        self.assertIn(pronto.Relationship("written_by"), pronto.Relationship._instances.values())
        self.assertIn(pronto.Relationship('written_by'), obo['ELO:0130001'].relations)
        self.assertIn(pronto.Relationship('has_written'), obo['ELO:0330001'].relations)


class TestProntoLocalOntology(TestProntoOntology):

    def test_local_owl_noimports(self):
        """Try to import a local owl ontology without its imports
        """
        owl = pronto.Ontology("tests/resources/cl.ont.gz", False)
        self.check_ontology(owl)

    def test_local_obo_noimports(self):
        """Try to import a local obo ontology without its imports
        """
        obo = pronto.Ontology("tests/resources/cmo.obo", False)
        self.check_ontology(obo)

    def test_local_owl_imports(self):
        """Try to import a local owl ontology with its imports
        """
        owl = pronto.Ontology("tests/resources/cl.ont.gz")
        self.check_ontology(owl)

    def test_local_obo_imports(self):
        """Try to import a local owl ontology with its imports
        """
        obo = pronto.Ontology("tests/resources/cmo.obo")
        self.check_ontology(obo)


class TestProntoRemoteOntology(TestProntoOntology):

    def test_remote_obo_noimports(self):
        """Try to import a remote obo ontology without its imports
        """
        obo = pronto.Ontology("http://purl.obolibrary.org/obo/pdumdv.obo", False)
        self.check_ontology(obo)

    def test_remote_owl_noimports(self):
        """Try to import a remote owl ontology without its imports
        """
        owl = pronto.Ontology("https://www.ebi.ac.uk/ols/ontologies/flu/download", False, parser="OwlXMLTreeParser")
        self.check_ontology(owl)

    def test_remote_obo_imports(self):
        """Try to import a remote obo ontology with its imports
        """
        obo = pronto.Ontology("http://purl.obolibrary.org/obo/doid.obo")
        self.check_ontology(obo)

    def test_remote_owl_imports(self):
        """Try to import a remote owl ontology with its imports
        """
        owl = pronto.Ontology("ftp://ftp.xenbase.org/pub/XenopusAnatomyOntology/xenopus_anatomy.owl")
        self.check_ontology(owl)


def setUpModule():
    warnings.simplefilter('ignore')

def tearDownModule():
    warnings.simplefilter(warnings.defaultaction)
